#include <stdio.h>
#include <libxml/parser.h>

#include <assert.h>

#include <GDD/arch_xml.h>
#include <GDD/ddg_xml.h>
#include <RS/greedy.h>

int main(int argc, char *argv[]) {
  // initialise XML parser
  xmlInitParser();

  // read architecture
  GDD_Arch *arch = GDD_arch_of_xml_file("arch.xml");
  assert(arch != NULL);
      
  // read ddg (ignore interloop dependency)
  SCEDA_Graph *dag = GDD_of_xml_file(TRUE, arch, "ddg.xml",NULL);
  assert(dag != NULL);

  // clean up XML parser
  xmlCleanupParser();

  // create the RS_DDAG
  GDD_DAG *ddg = GDD_dag_create(dag); 
  assert(ddg != NULL);

  // iterate over all types of the architecture
  GDD_ArchTypesIterator all_types;
  GDD_arch_types_iterator_init(arch, &all_types);
  while(GDD_arch_types_iterator_has_next(&all_types)) {
    char *type = GDD_arch_types_iterator_next(&all_types);
	
    fprintf(stdout,"Approximated register saturation for values of type %s is %d.\n", type, RS_ddag_estimate_saturation(ddg, type,TRUE));
  }
  GDD_arch_types_iterator_cleanup(&all_types);

  // clear the RS_DDAG
  GDD_dag_delete(ddg);

  SCEDA_graph_delete(dag);
  GDD_arch_delete(arch);
      
  return 0;
}
