#include <GDD/ddg.h>
#include <SIRA/loop.h>
#include <SIRA/solver.h>

int main(int argc, char *argv[]) {
  // two operations
  GDD_Opcode *i1 =  GDD_opcode_create("inst_1", 1, 0);
  GDD_opcode_extend_delta_w(i1, "GR", 0);

  GDD_Opcode *i2 =  GDD_opcode_create("inst_2", 1, 0);
  GDD_opcode_extend_delta_w(i2, "GR", 0);

  GDD_Opcode *i3 =  GDD_opcode_create("inst_3", 1, 0);
  GDD_opcode_extend_delta_w(i3, "GR", 0);

  GDD_Opcode *i4 =  GDD_opcode_create("inst_4", 1, 0);
  GDD_opcode_extend_delta_w(i4, "GR", 0);

  GDD_Opcode *i5 =  GDD_opcode_create("inst_5", 1, 0);
  GDD_opcode_extend_delta_w(i5, "GR", 0);

  GDD_Opcode *i6 =  GDD_opcode_create("inst_6", 1, 0);
  GDD_opcode_extend_delta_w(i6, "BR", 0);

  GDD_Opcode *i7 =  GDD_opcode_create("inst_7", 1, 0);

  // build the architecture description
  GDD_Arch *arch = GDD_arch_create();
  GDD_arch_add_regtype(arch, "GR", 61);
  GDD_arch_add_regtype(arch, "BR", 8);
  GDD_arch_add_opcode(arch, i1);
  GDD_arch_add_opcode(arch, i2);
  GDD_arch_add_opcode(arch, i3);
  GDD_arch_add_opcode(arch, i4);
  GDD_arch_add_opcode(arch, i5);
  GDD_arch_add_opcode(arch, i6);
  GDD_arch_add_opcode(arch, i7);

  // build a DDG
  SCEDA_Graph *g = SCEDA_graph_create((SCEDA_delete_fun)GDD_node_label_delete, (SCEDA_delete_fun)GDD_edge_label_delete); 

  SCEDA_Vertex *v1 = SCEDA_graph_add_vertex(g, GDD_node_label_create(1, i1));
  SCEDA_Vertex *v2 = SCEDA_graph_add_vertex(g, GDD_node_label_create(2, i2));
  SCEDA_Vertex *v3 = SCEDA_graph_add_vertex(g, GDD_node_label_create(3, i3));
  SCEDA_Vertex *v4 = SCEDA_graph_add_vertex(g, GDD_node_label_create(4, i4));
  SCEDA_Vertex *v5 = SCEDA_graph_add_vertex(g, GDD_node_label_create(5, i5));
  SCEDA_Vertex *v6 = SCEDA_graph_add_vertex(g, GDD_node_label_create(6, i6));
  SCEDA_Vertex *v7 = SCEDA_graph_add_vertex(g, GDD_node_label_create(7, i7));

  SCEDA_graph_add_edge(g, v1, v2, GDD_edge_label_create("GR", "flowdep_reg", 3, 0));
  SCEDA_graph_add_edge(g, v1, v7, GDD_edge_label_create(NULL, "serial", 0, 0));
  SCEDA_graph_add_edge(g, v2, v3, GDD_edge_label_create("GR", "flowdep_reg", 3, 0));
  SCEDA_graph_add_edge(g, v3, v3, GDD_edge_label_create("GR", "flowdep_reg", 1, 1));
  SCEDA_graph_add_edge(g, v3, v7, GDD_edge_label_create("GR", "flowdep_reg", 1, 0));      
  SCEDA_graph_add_edge(g, v4, v4, GDD_edge_label_create("GR", "flowdep_reg", 1, 1));
  SCEDA_graph_add_edge(g, v4, v6, GDD_edge_label_create("GR", "flowdep_reg", 1, 0));
  SCEDA_graph_add_edge(g, v5, v1, GDD_edge_label_create(NULL, "serial", 1, 4));
  SCEDA_graph_add_edge(g, v5, v5, GDD_edge_label_create("GR", "flowdep_reg", 1, 1));
  SCEDA_graph_add_edge(g, v6, v7, GDD_edge_label_create("BR", "flowdep_reg", 3, 0));

  // create a siraloop with MIIrec = 2
  SIRA_Loop *loop = SIRA_loop_create(g, 2);

  // initialise settings for the solver
  SIRA_Param settings;
  SIRA_param_init(&settings);
  SIRA_param_set_ual_flag(&settings, FALSE);
  SIRA_param_set_bounded_flag(&settings, TRUE);
  SIRA_param_set_check_flag(&settings, TRUE);

  SIRA_WeightMap *weight_map = SIRA_weight_map_create();
  SIRA_weight_map_set_weight_of_type(weight_map, "GR", 1);

  SCEDA_HashMap *solution;
  if(SIRA_solver_single(loop, &settings, weight_map, 2, &solution) == 0) {
    fprintf(stdout,"siralina found a solution\n");
    if(SIRA_param_get_ual_flag(&settings)) {
      fprintf(stdout,"UAL semantics\n");
    } else {
      fprintf(stdout,"NUAL semantics\n");
    }
    SCEDA_hashmap_delete(solution);
  }

  // clean the settings (free some internal data structures)
  SIRA_param_cleanup(&settings);

  SIRA_weight_map_delete(weight_map);

  SIRA_loop_delete(loop); // delete the loop

  SCEDA_graph_delete(g); // delete g (not that reuse decisions must be unapplied before this call!)
  GDD_arch_delete(arch);     // and the architecture

  return 0;
}
