/*
   Copyright Universite de Versailles Saint-Quentin en Yvelines 2009
   AUTHORS: Sebastien Briais, Sid-Ahmed-Ali Touati

   This file is part of SIRA.
   
   SIRA is free software: you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.
   
   SIRA is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with SIRA.  If not, see
   <http://www.gnu.org/licenses/>.
*/

/** \mainpage
    
    \section intro_sec Introduction

    This library implements SiraLina, a powerful heuristic that solves
    the SIRA problem.

    It is based on the research article <em>SIRALINA: Efficient Method
    for Periodic Task Scheduling with Storage Requirement
    Minimisation</em> by Karine Deschinkel and Sid-Ahmed-Ali Touati.

    \section dep_sec Required libraries

    \li For data structures and graph algorithms, SCEDA library is
    required.

    \li For manipulation of data dependency graphs, DDG is required.

    \li SIRA includes the "hungarian" library of Cyrill Stachniss,
    which is in the public domain.

    This library offers actually three different implementations of siralina:

    \li A standalone version that does not require more libraries (it
    is based on a min cost flow algorithm)

    \li A version that additionnally requires LPSolve <http://lpsolve.sourceforge.net/5.5/> 

    \li A version that additionnally requires GLPK <http://www.gnu.org/software/glpk/>

    It is possible to chose at compile time which version to
    build. Note that if you chose to use the GLPK version, it will
    taint the resulting binary archive file with the GPLv3.

    For efficiency reasons, we advise to use the GLPK version.

    \section install_dec Installation

    To install SIRA, please refer to INSTALL file of the distribution.

    \section compil_sec Using SIRA library

    \li To compile a project that uses the SIRA library, make
    sure to add SIRA to your include path with -I flag of your C
    compiler.

    \li To link a project against the SIRA library, make sure to
    add SIRA to your library path with -L flag of your C
    compiler.<br/> Then link with "-lSIRA -lDDG -lSCEDA" flags.

    Depending on the way SIRA was built, you may also need to link
    against GLPK (add "-lgplk" to the link flags) or LPSolve (add
    "-llpsolve55" to the link flags).

    Please refer to the example folder for further details.

    \section content_sec Content of the library

    \li \ref sira

    \section prog_notes Notes to programmers

    All the functions, macros, type definitions of SIRA library are
    prefixed by "SIRA_" (or "_SIRA_", "__SIRA_"). There is an
    exception for symbols coming from the hungarian library which are
    all prefixed with "hungarian_".

    Since SIRA depends on DDG, it relies heavily on SCEDA library for
    representing its data structures. In particular, it makes use of
    SCEDA_List, SCEDA_HashSet, SCEDA_HashMap and of course SCEDA_Graph
    data structures.  To change the internal representation of DDG,
    you will <b>in theory</b> just need to provide a SCEDA compatible
    API to manipulate all these data structures (and also modify
    libDDG to reflect these changes).

    \section history_sec History

    \subsection v110 v 1.1.0

    \li Upgraded to GDD v 1.1.0

    \subsection v100 v 1.0.0

    \li Initial release

    \section license_sec License

    Copyright Universite de Versailles Saint-Quentin en Yvelines 2009

    AUTHORS: Sebastien Briais, Sid-Ahmed-Ali Touati

    SIRA is free software: you can redistribute it and/or modify it
    under the terms of the GNU Lesser General Public License as
    published by the Free Software Foundation, either version 3 of the
    License, or (at your option) any later version.
    
    SIRA is distributed in the hope that it will be useful, but
    WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
    Lesser General Public License for more details.
    
    You should have received a copy of the GNU Lesser General Public
    License along with SIRA.  If not, see
    <http://www.gnu.org/licenses/>.

*/

    
