/*
   Copyright Universite de Versailles Saint-Quentin en Yvelines 2009
   AUTHORS: Sebastien Briais, Sid-Ahmed-Ali Touati

   This file is part of SIRA.
   
   SIRA is free software: you can redistribute it and/or modify it
   under the terms of the GNU Lesser General Public License as
   published by the Free Software Foundation, either version 3 of the
   License, or (at your option) any later version.
   
   SIRA is distributed in the hope that it will be useful, but
   WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
   
   You should have received a copy of the GNU Lesser General Public
   License along with SIRA.  If not, see
   <http://www.gnu.org/licenses/>.
*/
#include "stringutil.h"

#include "regconstraints.h"
#include "boxed.h"

#include <limits.h>

/** Create an empty register constraints map.

    @return a new register constraints map.
*/
SIRA_RegConstraintsMap *SIRA_regconstraints_map_create() {
  return SCEDA_hashmap_create((SCEDA_delete_fun)free, (SCEDA_delete_fun)boxed_delete, (SCEDA_match_fun)string_match, (SCEDA_hash_fun)string_hash);
}

/** Set register constraints of given type.

    @param[in] regconstraints_map = register constraints map
    @param[in] type = type
    @param[in] constraint = maximal number of register of given type

    @return 0 in case of success, <>0 otherwise */
int SIRA_regconstraints_map_set_constraint_of_type(SIRA_RegConstraintsMap *regconstraints_map, const char *type, int constraint) {
  if(!SCEDA_hashmap_contains_key(regconstraints_map, type)) {
    return SCEDA_hashmap_put(regconstraints_map, safe_strdup(type), boxed_create(int, constraint), NULL);
  }
  return -1;
}

/** Set register constraints of given type.

    @param[in] regconstraints_map = register constraints map
    @param[in] type = type

    @return constraint associated to given type */
int SIRA_regconstraints_map_get_constraint_of_type(SIRA_RegConstraintsMap *regconstraints_map, const char *type) {
  boxed(int) w = SCEDA_hashmap_get(regconstraints_map, type);
  if(w == NULL) {
    return INT_MAX;
  }
  return boxed_get(w);
}

